(function () {
    'use strict';

    angular
        .module('app.service')
        .factory('currentCityService', currentCityService);

    currentCityService.$inject = ['$q', '$timeout', '$rootScope', 'localStorageService', 'httpService'];

    function currentCityService ($q, $timeout, $rootScope, localStorageService, httpService) {
        var city_key = 'current_city';
        var gaodeScript, provinces, cities = {};
        var position_option = {
            enableHighAccuracy: true,
            maximumAge:  3600000,  // 1 hour
            timeout: 10000
        };
        var service = {
            fetchCities: fetchCities,
            setCurrentCity: setCurrentCity,
            setCurrentCityByName: setCurrentCityByName,
            getCurrentCity: getCurrentCity,
            clearCurrentCity: clearCurrentCity,
            locateByGPS: locateByGPS
        };

        $rootScope.$on('auth:signout', clearCurrentCity);

        init();

        return service;

        ////////////////
        function init() {
            fetchCities().then(function (resp) {
                resp.data.provinceListWithCities.forEach(function (province) {
                    province.cities.forEach(function (city) {
                        cities[city.name] = city;
                    });
                });
            });
        }

        function fetchCities() {
            if (!provinces) {
                provinces = httpService.get('province_list_with_cities');
            }
            return provinces
        }

        function getCurrentCity() {
            return localStorageService.getObject(city_key);
        }

        function setCurrentCity(city) {
            localStorageService.setObject(city_key, city);
            $rootScope.$broadcast('currentCity:changed');
        }

        function setCurrentCityByName(cityName) {
            if (cityName in cities) {
                setCurrentCity(cities[cityName]);
            } else {
                debug('city not found by name: ' + cityName);
            }
        }

        function clearCurrentCity() {
            setCurrentCity(null);
        }

        function loadGaodeScript () {
            // asynchronously load gaode script
            if (!gaodeScript) {
                var deferred = $q.defer();
                var script = document.createElement('script');
                script.src = window.ArkenstoneAppSettings.gaodeScriptUrl +
                    '&key=' + window.ArkenstoneAppSettings.gaodeApiKey;
                document.body.appendChild(script);

                script.onload = script.onreadystatechange = function (e) {
                    $timeout(function () {
                        deferred.resolve(e);
                    });
                };

                script.onerror = function (e) {
                    $timeout(function () {
                        deferred.reject(e);
                    });
                };

                gaodeScript = deferred.promise;
            }
            return gaodeScript;
        }

        function locateByGPS() {
            var deferred = $q.defer();
            document.addEventListener("deviceready", function () {
                if ('geolocation' in navigator) {
                    navigator.geolocation.getCurrentPosition(function (position) {
                        // now that we get GPS coordinates, load gaode stuff
                        loadGaodeScript().then(function () {
                            AMap.service(['AMap.Geocoder'], function () {
                                var geocoder = new AMap.Geocoder({
                                    extensions: 'all'
                                });
                                var point = new AMap.LngLat(position.coords.longitude, position.coords.latitude);
                                geocoder.getAddress(point, function (status, result) {
                                    if (status === 'complete' && result.info == 'OK') {
                                        var addr = result.regeocode.addressComponent;
                                        var city = Boolean(addr.city) ? addr.city: addr.province;
                                        // remove trailing '市'
                                        if (city.match(/市$/)) {  // safari doesn't support String.prototype.endsWith
                                            city = city.slice(0, city.length - 1);
                                        }
                                        deferred.resolve(city);
                                    } else {
                                        deferred.reject('Gaode cannot geocode coordinate ' + point + ': ' + status);
                                    }
                                });
                            });
                        });
                    }, function (error) {
                        deferred.reject(error.message);
                    }, position_option);
                } else {
                    deferred.reject('navigator.geolocation not available');
                }
            }, false);

            deferred.promise.catch(function (reason) {
                debug('Geolocation error: ' + reason);
                return reason;
            });
            return deferred.promise;
        }

        function debug (message) {
            if (window.ArkenstoneAppSettings.debug) {
                console.log(message);
            }
        }
    }

})();
